﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Docking;

namespace RadDockingWPF_1079123
{
    public class CustomFlyoutBehavior : IFlyoutBehavior
    {
        private RadPane nextPaneToActivate;

        void IFlyoutBehavior.OnMouseEnter(IFlyoutHost host, RadPane targetPane)
        {
            host.SetSelectedPane(targetPane);

            if (host.CurrentState == FlyoutState.Closed)
            {
                var pane = host.SelectedPane;
                host.StartOpenTimer();
            }
            else if (host.CurrentState == FlyoutState.ClosingTimer)
            {
                host.CancelCloseTimer();
            }
        }

        void IFlyoutBehavior.OnMouseLeave(IFlyoutHost host)
        {
            var selectedPane = host.SelectedPane;

            if (selectedPane != null && !selectedPane.IsActive)
            {
                if (host.CurrentState == FlyoutState.Opened || host.CurrentState == FlyoutState.OpeningAnimation)
                {
                    host.StartCloseTimer();
                }
                else if (host.CurrentState == FlyoutState.OpeningAnimation)
                {
                    host.StartCloseAnimation();
                }
                else if (host.CurrentState == FlyoutState.OpeningTimer)
                {
                    host.CancelOpenTimer();
                    host.SetSelectedPane(null);
                }
            }
        }

        void IFlyoutBehavior.OnOpeningTimerTimeout(IFlyoutHost host)
        {
            host.StartOpenAnimation();
        }

        void IFlyoutBehavior.OnClosingTimerTimeout(IFlyoutHost host)
        {
            host.StartCloseAnimation();
        }

        void IFlyoutBehavior.OnPaneActivated(IFlyoutHost host, RadPane targetPane)
        {
            host.SetSelectedPane(targetPane);

            if (host.CurrentState == FlyoutState.Closed)
            {
                host.StartOpenAnimation();
            }
            else if (host.CurrentState == FlyoutState.ClosingTimer)
            {
                host.CancelCloseTimer();
            }
        }

        void IFlyoutBehavior.OnPaneDeactivated(IFlyoutHost host, RadPane targetPane)
        {
            var selectedPane = host.SelectedPane;

            if (selectedPane != null && !selectedPane.IsActive)
            {
                if (host.CurrentState == FlyoutState.Opened || host.CurrentState == FlyoutState.OpeningAnimation)
                {
                    if (this.nextPaneToActivate == null || (targetPane != null && this.nextPaneToActivate.Parent != targetPane.Parent))
                    {
                        host.StartCloseAnimation();
                    }
                }
                else if (host.CurrentState == FlyoutState.OpeningTimer)
                {
                    host.CancelOpenTimer();
                    host.SetSelectedPane(null);
                }
            }
        }

        void IFlyoutBehavior.OnPaneMouseLeftButtonDown(IFlyoutHost host, RadPane targetPane)
        {
            this.nextPaneToActivate = targetPane;
            host.ActivatePane(targetPane);
        }
    }
}
